class TrendsAPI:
    '''
    Trends related API endpoints.
    '''
    def __init__(self, api):
        self.api = api

    async def tags(self, **query):
        '''
        Tags that are being used more frequently within the past week.
        https://docs.joinmastodon.org/methods/trends/#tags
        '''
        path = '/api/v1/trends/tags'
        return await self.api._get(path, query=query)

    async def statuses(self, **query):
        '''
        Statuses that have been interacted with more than others.
        https://docs.joinmastodon.org/methods/trends/#statuses
        '''
        path = '/api/v1/trends/statuses'
        return await self.api._get(path, query=query)

    async def links(self, **query):
        '''
        Links that have been shared more than others.
        https://docs.joinmastodon.org/methods/trends/#links
        '''
        path = '/api/v1/trends/links'
        return await self.api._get(path, query=query)



class ListsAPI:
    '''
    List related API endpoints.
    '''
    def __init__(self, api):
        self.api = api

    async def all(self):
        '''
        Fetch all lists that the user owns.
        https://docs.joinmastodon.org/methods/lists/#get
        '''
        path = '/api/v1/lists'
        return await self.api._get(path)

    async def get(self, id):
        '''
        Fetch the list with the given ID.
        https://docs.joinmastodon.org/methods/lists/#get-one
        '''
        path = f'/api/v1/lists/{id}'
        return await self.api._get(path)

    async def create(self, title, replies_policy='list'):
        '''
        Create a new list.
        https://docs.joinmastodon.org/methods/lists/#create
        '''
        path = '/api/v1/lists'
        query = {'title': title, 'replies_policy': replies_policy}
        return await self.api._post(path, query=query)

    async def update(self, id, **query):
        '''
        Change the title of a list, or which replies to show.
        https://docs.joinmastodon.org/methods/lists/#update
        '''
        path = f'/api/v1/lists/{id}'
        return await self.api._put(path, query=query)

    async def delete(self, id):
        '''
        Delete a list.
        https://docs.joinmastodon.org/methods/lists/#delete
        '''
        path = f'/api/v1/lists/{id}'
        return await self.api._delete(path)

    async def accounts(self, id, **query):
        '''
        View accounts in a list.
        https://docs.joinmastodon.org/methods/lists/#accounts
        '''
        path = f'/api/v1/lists/{id}/accounts'
        return await self.api._get(path, query=query)

    async def add_accounts(self, id, *account_ids):
        '''
        Add accounts to the given list.
        Note that the user must be following these accounts.
        https://docs.joinmastodon.org/methods/lists/#accounts-add
        '''
        path = f'/api/v1/lists/{id}/accounts'
        query = {'account_ids': account_ids}
        return await self.api._post(path, query=query)

    async def remove_accounts(self, id, *account_ids):
        '''
        Remove accounts from the given list.
        https://docs.joinmastodon.org/methods/lists/#accounts-add
        '''
        path = f'/api/v1/lists/{id}/accounts'
        query = {'account_ids': account_ids}
        return await self.api._delete(path, query=query)



class NotificationsAPI:
    '''
    Notifications related API endpoints.
    '''
    def __init__(self, api):
        self.api = api

    async def all(self, **query):
        '''
        Notifications concerning the user.
        https://docs.joinmastodon.org/methods/notifications/#get
        '''
        path = '/api/v1/notifications'
        return await self.api._get(path, query=query)

    async def get(self, id, **query):
        '''
        View information about a notification with a given ID.
        https://docs.joinmastodon.org/methods/notifications/#get-one
        '''
        path = f'/api/v1/notifications/{id}'
        return await self.api._get(path, query=query)

    async def clear(self, **query):
        '''
        Clear all notifications from the server.
        https://docs.joinmastodon.org/methods/notifications/#clear
        '''
        path = '/api/v1/notifications/clear'
        return await self.api._post(path, query=query)

    async def dismiss(self, id, **query):
        '''
        Dismiss a single notification from the server.
        https://docs.joinmastodon.org/methods/notifications/#dismiss
        '''
        path = f'/api/v1/notifications/{id}/dismiss'
        return await self.api._post(path, query=query)



class TimelinesAPI:
    '''
    Timeline related API endpoints.
    '''
    def __init__(self, api):
        self.api = api

    async def public(self, **query):
        '''
        View public statuses.
        https://docs.joinmastodon.org/methods/timelines/#public
        '''
        path = '/api/v1/timelines/public'
        return await self.api._get(path, query=query)

    async def hashtag(self, tag, **query):
        '''
        View public statuses containing the given hashtag.
        https://docs.joinmastodon.org/methods/timelines/#tag
        '''
        path = f'/api/v1/timelines/tag/{tag}'
        return await self.api._get(path, query=query)

    async def home(self, **query):
        '''
        View statuses from followed users.
        https://docs.joinmastodon.org/methods/timelines/#home
        '''
        path = '/api/v1/timelines/home'
        return await self.api._get(path, query=query)

    async def list(self, id, **query):
        '''
        View statuses in the given list timeline.
        https://docs.joinmastodon.org/methods/timelines/#list
        '''
        path = f'/api/v1/timelines/list/{id}'
        return await self.api._get(path, query=query)
